#include <Wire.h>
#include <SD.h>
#include <SPI.h>
#include "RTClib.h"

//////////////// include libraries
#include <XBee.h>

#include "functions.h"

#include <Adafruit_Sensor.h>
#include <Adafruit_LSM303_U.h>
#include <Adafruit_L3GD20_U.h>
#include <Adafruit_9DOF.h>

// sensors
#define InThermPin 0
#define OutThermPin 1
#define Volts5Pin 2
#define Volts12Pin 3

#define SDChipSelectPin 4

#define StatusLEDPin 6

uint32_t last_transmit = 0;

#define trans_interval  1000

// initalize rtc
RTC_DS1307 rtc;

/* Assign a unique ID to the sensors */
Adafruit_9DOF                dof   = Adafruit_9DOF();
Adafruit_LSM303_Accel_Unified accel = Adafruit_LSM303_Accel_Unified(30301);
Adafruit_LSM303_Mag_Unified   mag   = Adafruit_LSM303_Mag_Unified(30302);
Adafruit_L3GD20_Unified       gyro  = Adafruit_L3GD20_Unified(20);

const int chipSelect = 4;

//////////////// initalize variables

#define PKT_MAX_LEN 60

// set XBee address of logger
#define Cmd_MY_Addr 0x0001

// set XBee address of this transmitter
#define White_MY_Addr 0x0002

// set XBee address of this transmitter
#define XBee_MY_Addr 0x0004

// PAN ID of xbee (must match all xbees)
#define XBee_PAN_ID  0x0B0B

// initalize an array to hold the data to send
uint8_t data[PKT_MAX_LEN];

// data definitions
sensors_event_t event;

int InTemp;
int OutTemp;
float Volt5Bus;
  
int Termistor(int RawADC) {
  float Temp;
  Temp = log(10000.0*((1024.0/RawADC-1)));
  Temp=1/(0.001129148+(0.000234125+(0.0000000876741*Temp*Temp))*Temp); //Temperature left in Kelvin
  Temp = Temp - 273.15;            // Convert Kelvin to Celcius
  Temp = (Temp * 9.0)/ 5.0 + 32.0; // Convert Celcius to Fahrenheit
  
  return (int)Temp;
}

float Meas5Volt(int RawADC){
  float Volts;
  
  // voltage divider
  float R1 = 8.06; // K
  float R2  = 10; // K
  
  Volts = 5.0 * RawADC / 1024.0;
  Volts *= (R1 + R2) / R2;
  
  return Volts;
}

void setup()
{
  Serial.begin(9600);
  pinMode(10, OUTPUT);
  SD.begin(SDChipSelectPin);
//  if (!SD.begin(SDChipSelectPin)) {            // Changed this from (10,11,12,13) to (chipSelect)
//    Serial.println(F("SD failed"));
//  }
//  
  accel.begin();
  mag.begin();
  gyro.begin();
//   if(!accel.begin() || !mag.begin() || !gyro.begin())
//  {
//    Serial.println(F("no sensor detected"));
//  }
    
  int XbeeStatus = InitXBee(XBee_MY_Addr, XBee_PAN_ID);
//  if(XbeeStatus){
//      Serial.print(F("Xbee failed, cde: "));
//      Serial.println(XbeeStatus); 
//    }
  
  // use only to set time once
  //rtc.adjust(DateTime(__DATE__, __TIME__));
  
    File dataFile = SD.open("datalog.csv", FILE_WRITE);

//  if (dataFile){
//    dataFile.print(F("millis,Temp,AccelXYZ,MagXYZ,GryoXYZ"));
//    dataFile.close();
//  }
  
}
void loop(void)
{
  // Thermistor sensor 
  InTemp=Termistor(analogRead(InThermPin));
  OutTemp=Termistor(analogRead(OutThermPin));
  
  // Voltage sensor
  Volt5Bus = Meas5Volt(analogRead(Volts5Pin));
  
    /* Calculate pitch and roll from the raw accelerometer data */
  accel.getEvent(&event);
  mag.getEvent(&event);
  gyro.getEvent(&event);
  
  DateTime now = rtc.now();

   uint8_t offset = 0;

   // start of data
    offset = addToTlm(millis(), data, offset);
    offset = addToTlm(now.unixtime(), data, offset);
    offset = addToTlm(InTemp, data, offset);
    offset = addToTlm(OutTemp, data, offset);
    offset = addToTlm(Volt5Bus, data, offset);
    offset = addToTlm(event.acceleration.x, data, offset);
    offset = addToTlm(event.acceleration.y, data, offset);
    offset = addToTlm(event.acceleration.z, data, offset);
    offset = addToTlm(event.magnetic.x, data, offset);
    offset = addToTlm(event.magnetic.y, data, offset);
    offset = addToTlm(event.magnetic.z, data, offset);
    offset = addToTlm(event.gyro.x, data, offset);
    offset = addToTlm(event.gyro.y, data, offset);
    offset = addToTlm(event.gyro.z, data, offset);
    // end of data

    if(millis() - last_transmit > trans_interval){
      last_transmit = millis();
    // send the data
    sendData(Cmd_MY_Addr, data, offset * sizeof(uint8_t));
    sendData(White_MY_Addr, data, offset * sizeof(uint8_t));
    }
  // read messages
  if (readMsg(data, 1) > 0) {
  
      // its a request for a HK packet
      if (data[4] == 0xAB && data[5] == 0xCD) {
        sentHKpkt(extract_uint16(data, 2));
      }
      // otherwise, log the data we received
      else {
        String data_Str = data2string(data);
        
        Serial.println(data_Str);
      }
    }

  String data_Str;
  data_Str = "";
  data_Str += millis();
  data_Str += ",";
  data_Str += now.unixtime();
  data_Str += ",";
  
  data_Str += InTemp; //display Fahrenheit
  data_Str += ",";  
  data_Str += OutTemp; //display Fahrenheit
  data_Str += ",";  
  data_Str += Volt5Bus;  
  
  data_Str += event.acceleration.x; 
  data_Str += ",";
  data_Str += event.acceleration.y; 
  data_Str += ",";
  data_Str += event.acceleration.z; 
  data_Str += ",";
  
  data_Str += event.magnetic.x; 
  data_Str += ",";
  data_Str += event.magnetic.y; 
  data_Str += ",";
  data_Str += event.magnetic.z; 
  data_Str += ",";

  data_Str += event.gyro.x; 
  data_Str += ",";
  data_Str += event.gyro.y; 
  data_Str += ",";
  data_Str += event.gyro.z; 

  Serial.println(data_Str);
  
  File dataFile = SD.open("datalog.csv", FILE_WRITE);

  if (dataFile){
    dataFile.print(data_Str);
    dataFile.close();
  }
  delay(100);
}



